import click
from cogeotiff.cog import create_cog

@click.command()
@click.argument('src_path')
@click.argument('dst_path')
@click.option('--overview-resampling', 
             default='nearest',
             help='resampling method for create pyramiddefault: nearest'
             )
@click.option('--overview-level', 
             type=int, 
             default=None,
             help='levels to build'
             )
@click.option('--nodata', 
             type=int, 
             default=None,
             help='Assign a specified nodata value to output bands. default: 0'
             )
@click.option('--block-size', 
             type=int, 
             default=512,
             help='tiled size, default: 512'
             )
@click.option('--compress', 
             type=str, 
             default='raw',
             help='compress method,default: raw')
def create_cog_command(
    src_path,
    dst_path, 
    overview_resampling, 
    overview_level, 
    nodata,
    block_size,
    compress
):
    create_cog(
    src_path,
    dst_path, 
    overview_resampling=overview_resampling, 
    overview_level=overview_level, 
    nodata=nodata,
    block_size=block_size,
    compress=compress)


from osgeo import gdal

def get_max_overview(src_path, block_size=512):
    image = gdal.Open(src_path)
    width = image.RasterXSize
    height = image.RasterYSize

    nlevel = 0
    overview = 1

    while min(width // overview, height // overview) > block_size:
        overview *= 2
        nlevel += 1

    return nlevel


def check_overview(ds):
    '''check dataset overview exit or not.
     If overview exit (internel or extrial), return True, else False.
    
    Arguments:
        ds {gdal.Dataset} -- opened gdal raster dataset
    
    Returns:
        Boolen -- True for overview exit, False for overview not exit.
    '''

    main_band = ds.GetRasterBand(1)
    ovr_count = main_band.GetOverviewCount()
    if ovr_count == 0:
        return False
    else:
        return True


import click
from .create import create_cog_command
from .validate import validate_cog_command

@click.group()
def entry_point():
    '''Cloud Optimized GeoTiff tools for create COG file and validate COG file.

\b 
    command:
        [cog create] : create Cloud Optimized GeoTiff.
        [cog validate]: validate the cog file is valid.
\b
    for detial help, check [cog create --help] or [cog validate --help]
    '''
    pass

entry_point.add_command(create_cog_command, name='create')
entry_point.add_command(validate_cog_command, name='validate')


import os
from osgeo import gdal
import subprocess
from .profiles import cog_profile
from .utils import check_overview, get_max_overview


def create_cog(
    src_path, 
    dst_path, 
    overview_resampling='nearest', 
    overview_level=None, 
    nodata=0,
    block_size=512,
    compress='raw',
    **kwargs):
    '''create cog file from a specific file(tiff img hdr...)
    
    Arguments:
        src_path {string} -- source file path
        dst_path {string} -- target file path
    
    Keyword Arguments:
        overview_resampling {str} -- resampling method for create pyramid (default: {'nearest'})
        overview_level {int} -- levels to build (default: {None})
        nodata {float} -- Assign a specified nodata value to output bands (default: 0)
        block_size {int} -- tiled size (default: {512})
        compress {str} -- compress method (default: {'raw'})
    '''
    dataset = gdal.Open(src_path)

    # overview level:
    if not overview_level:
        nlevel = get_max_overview(src_path)
    overview_list = [2 ** j for j in range(1, nlevel + 1)]
    overview_list = ' '.join([str(x) for x in overview_list])
    
    # check overview extis or not, if overview exits, then skip create overview
    if not check_overview(dataset):
        print('++++creating overview++++')
        overview_command = ' '.join(['gdaladdo', '-ro', '-r', overview_resampling, src_path, overview_list ])
        addo_result = subprocess.run(overview_command, shell=True, check=True)
    else:
        print('++++overview already exits, skip create overview++++')

    # translate the file to tiled tif
    print('\n++++generate cloud optimized geotiff++++')
    out_profile = cog_profile.get(compress)
    out_profile.update({'nodata':nodata, 'block_size':block_size})
    out_profile = dict(out_profile).values()
    print(out_profile)
    translate_command = ' '.join(['gdal_translate', ' '.join(out_profile), src_path, dst_path
        ])
    subprocess.run(translate_command, shell=True)

    # remove the external ovr file
    # os.remove(src_path+'.ovr')

if __name__ == "__main__":
    src_path = '/Volumes/sshuair/imagery-path/cog-test/test_7.tif'
    dest_path = '/Volumes/sshuair/imagery-path/cog-test/cache/test_7.tiff.cog'
    create_cog(src_path, dest_path, compress='jpeg')


#coding: --utf-8--
from setuptools import setup

# install requirements
inst_reqs = [
    'click', 'gdal'
]

# readme
with open('README.md', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='cogeotiff',
    version = '0.1.1',
    description = u'Cloud Optimized GeoTiff utility tools.',
    long_description = readme,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3',
        'Topic :: Multimedia :: Graphics :: Graphics Conversion',
        'Topic :: Scientific/Engineering :: GIS'],
    keywords='GeoTiff COG GDAL',
    author='sshuair',
    author_email='sshuair@gmail.com',
    url='https://github.com/sshuair/cogeotiff',
    packages = ['cogeotiff', 'cogeotiff.cli'],
    install_requires=inst_reqs,
    entry_points='''
        [console_scripts]
        cog=cogeotiff.cli.cli:entry_point
    '''
)



from collections import UserDict


class Profile(UserDict):
    defaults = {}

    def __init__(self, data={}, **kwargs):
        UserDict.__init__(self)
        initdata = self.defaults.copy()
        initdata.update(data)
        initdata.update(**kwargs)
        self.data.update(initdata)

    def __getitem__(self, key):
        return self.data[key]
    
    def __setitem__(self, key, value):
        #TODO: not good method, add kwargs
        if key == 'driver':
            self.data[key] = '-of {}'.format(value)
        elif key == 'tiled':
            self.data[key] = '-co TILED={}'.format('YES' if value else 'NO')
        elif key == 'block_size':
            self.data[key] = '-co BLOCKXSIZE={value} -co BLOCKYSIZE={value}'.format(value=value)
        elif key == 'copy_src_overviews':
            self.data[key] = '-co COPY_SRC_OVERVIEWS={}'.format('YES' if value else 'NO')
        elif key == 'nodata':
            self.data[key] = '-a_nodata {}'.format(value)
        elif key == 'compress':
            self.data[key] = '-co COMPRESS={}'.format(value)
        elif key == 'photometric':
            self.data[key] = '-co PHOTOMETRIC={}'.format(value)
        else:
            raise KeyError('not support key {key}'.format(key=key))


class RAWProfile(Profile):
    '''no compress GTiff'''
    
    defaults = {
        'driver': 'GTiff',
        'tiled': 'YES',
        'block_size': 256,
        'copy_src_overviews': True, 
        'nodata': 0,
    }


class JPEGProfile(Profile):
    defaults = {
        'driver': 'GTiff',
        'tiled': True,
        'block_size': 256,
        'copy_src_overviews': True, 
        'nodata': 0,
        'compress': 'JPEG',
        "photometric": "YCBCR",
    }


class WEBPProfile(Profile):
    """Tiled, pixel-interleaved, WEBP-compressed, 8-bit GTiff."""

    defaults = {
        "driver": "GTiff",
        "tiled": True,
        "block_size": 512,
        'copy_src_overviews': True, 
        'nodata': 0,
        "compress": "WEBP",
    }


class ZSTDProfile(Profile):
    """Tiled, pixel-interleaved, ZSTD-compressed GTiff.
    Note: ZSTD compression is available since gdal 2.3
    """

    defaults = {
        "driver": "GTiff",
        "tiled": True,
        "block_size": 512,
        'copy_src_overviews': True, 
        'nodata': 0,
        'dtype': 'Byte',
        "compress": "ZSTD",
    }


class LZWProfile(Profile):
    """Tiled, pixel-interleaved, LZW-compressed GTiff."""

    defaults = {
        "driver": "GTiff",
        "tiled": True,
        "block_size": 512,
        'copy_src_overviews': True, 
        'nodata': 0,
        "compress": "LZW",
    }


class DEFLATEProfile(Profile):
    """Tiled, pixel-interleaved, DEFLATE-compressed GTiff."""

    defaults = {
        "driver": "GTiff",
        "tiled": True,
        "block_size": 512,
        'copy_src_overviews': True, 
        'nodata': 0,
        "compress": "DEFLATE",
    }


class PACKBITSProfile(Profile):
    """Tiled, pixel-interleaved, PACKBITS-compressed GTiff."""

    defaults = {
        "driver": "GTiff",
        "tiled": True,
        "block_size": 512,
        'copy_src_overviews': True, 
        'nodata': 0,
        "compress": "PACKBITS",
    }


class COGProfiles(dict):
    def __init__(self):
        self.update({
            'jpeg': JPEGProfile(),
            'raw': RAWProfile(),
            'deflate': DEFLATEProfile()
        })

    def get(self, key):
        if key not in self.keys():
            raise KeyError('{} is not in COG profile'.format(key))
        
        return self[key].copy()

cog_profile = COGProfiles()
